require("tiny-debug")
require("utils")
require("tiny-tools")

local tiny = require("lib/tiny")
World = tiny.world()

require("generated/filter-types")
require("generated/assets")
require("generated/all-systems")

local scenarios = {
    default = function()
        -- TODO: Add default entities
    end,
    textTestScenario = function()
        World:addEntity({
            position = { x = 0, y = 600 },
            drawAsText = {
                text = "Hello, world!",
                style = TextStyle.Inverted,
            },
            velocity = { x = 240, y = -500 },
            mass = 1,
            decayAfterSeconds = 10,
        })
    end,
}

local currentScenario = scenarios.textTestScenario
local freeze = false
local delta

function love.load()
    currentScenario()
    love.graphics.setBackgroundColor(1, 1, 1)
    love.graphics.setFont(EtBt7001Z0xa(32))
end

function love.update(dt)
    delta = dt

    if love.keyboard.isDown("r") then
        World:clearEntities()
        currentScenario()
        freeze = false
    end

    if love.keyboard.isDown("f") then
        freeze = not freeze
    end

    if freeze then
        return
    end

    World:update(delta, function(_, system)
        if system.deferToEnd then
            return false
        end
        return not system.isDrawSystem
    end)
end

function love.draw()
    World:update(delta, function(_, system)
        if system.deferToEnd then
            return false
        end
        return system.isDrawSystem
    end)

    World:update(delta, function(_, system)
        return system.deferToEnd
    end)
end
